@@ -28,21 +28,19 @@ from mypy.stubgen import (
     Options,
     collect_build_targets,
     generate_stubs,
-    get_sig_generators,
     is_blacklisted_path,
     is_non_library_module,
     mypy_options,
     parse_options,
 )
-from mypy.stubgenc import (
-    generate_c_function_stub,
-    generate_c_property_stub,
-    generate_c_type_stub,
-    infer_method_args,
+from mypy.stubgenc import InspectionStubGenerator, infer_c_method_args
+from mypy.stubutil import (
+    ClassInfo,
+    common_dir_prefix,
     infer_method_ret_type,
-    is_c_property_readonly,
+    remove_misplaced_type_comments,
+    walk_packages,
 )
-from mypy.stubutil import common_dir_prefix, remove_misplaced_type_comments, walk_packages
 from mypy.test.data import DataDrivenTestCase, DataSuite
 from mypy.test.helpers import assert_equal, assert_string_arrays_equal, local_sys_path_set
 
@@ -62,7 +60,8 @@ class StubgenCmdLineSuite(unittest.TestCase):
                 os.mkdir(os.path.join("subdir", "pack"))
                 self.make_file("subdir", "pack", "__init__.py")
                 opts = parse_options(["subdir"])
-                py_mods, c_mods = collect_build_targets(opts, mypy_options(opts))
+                py_mods, pyi_mods, c_mods = collect_build_targets(opts, mypy_options(opts))
+                assert_equal(pyi_mods, [])
                 assert_equal(c_mods, [])
                 files = {mod.path for mod in py_mods}
                 assert_equal(
@@ -87,7 +86,8 @@ class StubgenCmdLineSuite(unittest.TestCase):
                 self.make_file("pack", "a.py")
                 self.make_file("pack", "b.py")
                 opts = parse_options(["-p", "pack"])
-                py_mods, c_mods = collect_build_targets(opts, mypy_options(opts))
+                py_mods, pyi_mods, c_mods = collect_build_targets(opts, mypy_options(opts))
+                assert_equal(pyi_mods, [])
                 assert_equal(c_mods, [])
                 files = {os.path.relpath(mod.path or "FAIL") for mod in py_mods}
                 assert_equal(
@@ -111,7 +111,7 @@ class StubgenCmdLineSuite(unittest.TestCase):
                 os.chdir(tmp)
                 self.make_file(tmp, "mymodule.py", content="import a")
                 opts = parse_options(["-m", "mymodule"])
-                py_mods, c_mods = collect_build_targets(opts, mypy_options(opts))
+                collect_build_targets(opts, mypy_options(opts))
                 assert captured_output.getvalue() == ""
             finally:
                 sys.stdout = sys.__stdout__
@@ -702,10 +702,14 @@ class StubgenPythonSuite(DataSuite):
         out_dir = "out"
         try:
             try:
-                if not testcase.name.endswith("_import"):
-                    options.no_import = True
-                if not testcase.name.endswith("_semanal"):
-                    options.parse_only = True
+                if testcase.name.endswith("_inspect"):
+                    options.inspect = True
+                else:
+                    if not testcase.name.endswith("_import"):
+                        options.no_import = True
+                    if not testcase.name.endswith("_semanal"):
+                        options.parse_only = True
+
                 generate_stubs(options)
                 a: list[str] = []
                 for module in modules:
@@ -724,11 +728,22 @@ class StubgenPythonSuite(DataSuite):
 
     def parse_flags(self, program_text: str, extra: list[str]) -> Options:
         flags = re.search("# flags: (.*)$", program_text, flags=re.MULTILINE)
+        pyversion = None
         if flags:
             flag_list = flags.group(1).split()
+            for i, flag in enumerate(flag_list):
+                if flag.startswith("--python-version="):
+                    pyversion = flag.split("=", 1)[1]
+                    del flag_list[i]
+                    break
         else:
             flag_list = []
         options = parse_options(flag_list + extra)
+        if pyversion:
+            # A hack to allow testing old python versions with new language constructs
+            # This should be rarely used in general as stubgen output should not be version-specific
+            major, minor = pyversion.split(".", 1)
+            options.pyversion = (int(major), int(minor))
         if "--verbose" not in flag_list:
             options.quiet = True
         else:
@@ -770,35 +785,28 @@ class StubgencSuite(unittest.TestCase):
     """
 
     def test_infer_hash_sig(self) -> None:
-        assert_equal(infer_method_args("__hash__"), [self_arg])
+        assert_equal(infer_c_method_args("__hash__"), [self_arg])
         assert_equal(infer_method_ret_type("__hash__"), "int")
 
     def test_infer_getitem_sig(self) -> None:
-        assert_equal(infer_method_args("__getitem__"), [self_arg, ArgSig(name="index")])
+        assert_equal(infer_c_method_args("__getitem__"), [self_arg, ArgSig(name="index")])
 
     def test_infer_setitem_sig(self) -> None:
         assert_equal(
-            infer_method_args("__setitem__"),
+            infer_c_method_args("__setitem__"),
             [self_arg, ArgSig(name="index"), ArgSig(name="object")],
         )
         assert_equal(infer_method_ret_type("__setitem__"), "None")
 
+    def test_infer_eq_op_sig(self) -> None:
+        for op in ("eq", "ne", "lt", "le", "gt", "ge"):
+            assert_equal(
+                infer_c_method_args(f"__{op}__"), [self_arg, ArgSig(name="other", type="object")]
+            )
+
     def test_infer_binary_op_sig(self) -> None:
-        for op in (
-            "eq",
-            "ne",
-            "lt",
-            "le",
-            "gt",
-            "ge",
-            "add",
-            "radd",
-            "sub",
-            "rsub",
-            "mul",
-            "rmul",
-        ):
-            assert_equal(infer_method_args(f"__{op}__"), [self_arg, ArgSig(name="other")])
+        for op in ("add", "radd", "sub", "rsub", "mul", "rmul"):
+            assert_equal(infer_c_method_args(f"__{op}__"), [self_arg, ArgSig(name="other")])
 
     def test_infer_equality_op_sig(self) -> None:
         for op in ("eq", "ne", "lt", "le", "gt", "ge", "contains"):
@@ -806,46 +814,31 @@ class StubgencSuite(unittest.TestCase):
 
     def test_infer_unary_op_sig(self) -> None:
         for op in ("neg", "pos"):
-            assert_equal(infer_method_args(f"__{op}__"), [self_arg])
+            assert_equal(infer_c_method_args(f"__{op}__"), [self_arg])
 
     def test_infer_cast_sig(self) -> None:
         for op in ("float", "bool", "bytes", "int"):
             assert_equal(infer_method_ret_type(f"__{op}__"), op)
 
-    def test_generate_c_type_stub_no_crash_for_object(self) -> None:
+    def test_generate_class_stub_no_crash_for_object(self) -> None:
         output: list[str] = []
         mod = ModuleType("module", "")  # any module is fine
-        imports: list[str] = []
-        generate_c_type_stub(
-            mod,
-            "alias",
-            object,
-            output,
-            imports=imports,
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
-        )
-        assert_equal(imports, [])
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+
+        gen.generate_class_stub("alias", object, output)
+        assert_equal(gen.get_imports().splitlines(), [])
         assert_equal(output[0], "class alias:")
 
-    def test_generate_c_type_stub_variable_type_annotation(self) -> None:
+    def test_generate_class_stub_variable_type_annotation(self) -> None:
         # This class mimics the stubgen unit test 'testClassVariable'
         class TestClassVariableCls:
             x = 1
 
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType("module", "")  # any module is fine
-        generate_c_type_stub(
-            mod,
-            "C",
-            TestClassVariableCls,
-            output,
-            imports=imports,
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
-        )
-        assert_equal(imports, [])
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_class_stub("C", TestClassVariableCls, output)
+        assert_equal(gen.get_imports().splitlines(), ["from typing import ClassVar"])
         assert_equal(output, ["class C:", "    x: ClassVar[int] = ..."])
 
     def test_generate_c_type_inheritance(self) -> None:
@@ -853,35 +846,19 @@ class StubgencSuite(unittest.TestCase):
             pass
 
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType("module, ")
-        generate_c_type_stub(
-            mod,
-            "C",
-            TestClass,
-            output,
-            imports=imports,
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
-        )
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_class_stub("C", TestClass, output)
         assert_equal(output, ["class C(KeyError): ..."])
-        assert_equal(imports, [])
+        assert_equal(gen.get_imports().splitlines(), [])
 
     def test_generate_c_type_inheritance_same_module(self) -> None:
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType(TestBaseClass.__module__, "")
-        generate_c_type_stub(
-            mod,
-            "C",
-            TestClass,
-            output,
-            imports=imports,
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
-        )
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_class_stub("C", TestClass, output)
         assert_equal(output, ["class C(TestBaseClass): ..."])
-        assert_equal(imports, [])
+        assert_equal(gen.get_imports().splitlines(), [])
 
     def test_generate_c_type_inheritance_other_module(self) -> None:
         import argparse
@@ -890,38 +867,22 @@ class StubgencSuite(unittest.TestCase):
             pass
 
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType("module", "")
-        generate_c_type_stub(
-            mod,
-            "C",
-            TestClass,
-            output,
-            imports=imports,
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
-        )
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_class_stub("C", TestClass, output)
         assert_equal(output, ["class C(argparse.Action): ..."])
-        assert_equal(imports, ["import argparse"])
+        assert_equal(gen.get_imports().splitlines(), ["import argparse"])
 
     def test_generate_c_type_inheritance_builtin_type(self) -> None:
         class TestClass(type):
             pass
 
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType("module", "")
-        generate_c_type_stub(
-            mod,
-            "C",
-            TestClass,
-            output,
-            imports=imports,
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
-        )
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_class_stub("C", TestClass, output)
         assert_equal(output, ["class C(type): ..."])
-        assert_equal(imports, [])
+        assert_equal(gen.get_imports().splitlines(), [])
 
     def test_generate_c_type_with_docstring(self) -> None:
         class TestClass:
@@ -931,22 +892,16 @@ class StubgencSuite(unittest.TestCase):
                 """
 
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType(TestClass.__module__, "")
-        generate_c_function_stub(
-            mod,
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_function_stub(
             "test",
             TestClass.test,
             output=output,
-            imports=imports,
-            self_var="self",
-            cls=TestClass,
-            class_name="TestClass",
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
+            class_info=ClassInfo(self_var="self", cls=TestClass, name="TestClass"),
         )
         assert_equal(output, ["def test(self, arg0: int) -> Any: ..."])
-        assert_equal(imports, [])
+        assert_equal(gen.get_imports().splitlines(), [])
 
     def test_generate_c_type_with_docstring_no_self_arg(self) -> None:
         class TestClass:
@@ -956,22 +911,16 @@ class StubgencSuite(unittest.TestCase):
                 """
 
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType(TestClass.__module__, "")
-        generate_c_function_stub(
-            mod,
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_function_stub(
             "test",
             TestClass.test,
             output=output,
-            imports=imports,
-            self_var="self",
-            cls=TestClass,
-            class_name="TestClass",
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
+            class_info=ClassInfo(self_var="self", cls=TestClass, name="TestClass"),
         )
         assert_equal(output, ["def test(self, arg0: int) -> Any: ..."])
-        assert_equal(imports, [])
+        assert_equal(gen.get_imports().splitlines(), [])
 
     def test_generate_c_type_classmethod(self) -> None:
         class TestClass:
@@ -980,22 +929,16 @@ class StubgencSuite(unittest.TestCase):
                 pass
 
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType(TestClass.__module__, "")
-        generate_c_function_stub(
-            mod,
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_function_stub(
             "test",
             TestClass.test,
             output=output,
-            imports=imports,
-            self_var="cls",
-            cls=TestClass,
-            class_name="TestClass",
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
+            class_info=ClassInfo(self_var="cls", cls=TestClass, name="TestClass"),
         )
-        assert_equal(output, ["@classmethod", "def test(cls, *args, **kwargs) -> Any: ..."])
-        assert_equal(imports, [])
+        assert_equal(output, ["@classmethod", "def test(cls, *args, **kwargs): ..."])
+        assert_equal(gen.get_imports().splitlines(), [])
 
     def test_generate_c_type_classmethod_with_overloads(self) -> None:
         class TestClass:
@@ -1008,19 +951,13 @@ class StubgencSuite(unittest.TestCase):
                 pass
 
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType(TestClass.__module__, "")
-        generate_c_function_stub(
-            mod,
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_function_stub(
             "test",
             TestClass.test,
             output=output,
-            imports=imports,
-            self_var="cls",
-            cls=TestClass,
-            class_name="TestClass",
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
+            class_info=ClassInfo(self_var="cls", cls=TestClass, name="TestClass"),
         )
         assert_equal(
             output,
@@ -1033,7 +970,7 @@ class StubgencSuite(unittest.TestCase):
                 "def test(cls, arg0: int) -> Any: ...",
             ],
         )
-        assert_equal(imports, ["from typing import overload"])
+        assert_equal(gen.get_imports().splitlines(), ["from typing import overload"])
 
     def test_generate_c_type_with_docstring_empty_default(self) -> None:
         class TestClass:
@@ -1043,22 +980,16 @@ class StubgencSuite(unittest.TestCase):
                 """
 
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType(TestClass.__module__, "")
-        generate_c_function_stub(
-            mod,
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_function_stub(
             "test",
             TestClass.test,
             output=output,
-            imports=imports,
-            self_var="self",
-            cls=TestClass,
-            class_name="TestClass",
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
+            class_info=ClassInfo(self_var="self", cls=TestClass, name="TestClass"),
         )
         assert_equal(output, ["def test(self, arg0: str = ...) -> Any: ..."])
-        assert_equal(imports, [])
+        assert_equal(gen.get_imports().splitlines(), [])
 
     def test_generate_c_function_other_module_arg(self) -> None:
         """Test that if argument references type from other module, module will be imported."""
@@ -1071,19 +1002,11 @@ class StubgencSuite(unittest.TestCase):
             """
 
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType(self.__module__, "")
-        generate_c_function_stub(
-            mod,
-            "test",
-            test,
-            output=output,
-            imports=imports,
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
-        )
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_function_stub("test", test, output=output)
         assert_equal(output, ["def test(arg0: argparse.Action) -> Any: ..."])
-        assert_equal(imports, ["import argparse"])
+        assert_equal(gen.get_imports().splitlines(), ["import argparse"])
 
     def test_generate_c_function_same_module(self) -> None:
         """Test that if annotation references type from same module but using full path, no module
@@ -1098,19 +1021,11 @@ class StubgencSuite(unittest.TestCase):
             """
 
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType("argparse", "")
-        generate_c_function_stub(
-            mod,
-            "test",
-            test,
-            output=output,
-            imports=imports,
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
-        )
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_function_stub("test", test, output=output)
         assert_equal(output, ["def test(arg0: Action) -> Action: ..."])
-        assert_equal(imports, [])
+        assert_equal(gen.get_imports().splitlines(), [])
 
     def test_generate_c_function_other_module(self) -> None:
         """Test that if annotation references type from other module, module will be imported."""
@@ -1121,19 +1036,11 @@ class StubgencSuite(unittest.TestCase):
             """
 
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType(self.__module__, "")
-        generate_c_function_stub(
-            mod,
-            "test",
-            test,
-            output=output,
-            imports=imports,
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
-        )
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_function_stub("test", test, output=output)
         assert_equal(output, ["def test(arg0: argparse.Action) -> argparse.Action: ..."])
-        assert_equal(set(imports), {"import argparse"})
+        assert_equal(gen.get_imports().splitlines(), ["import argparse"])
 
     def test_generate_c_function_same_module_nested(self) -> None:
         """Test that if annotation references type from same module but using full path, no module
@@ -1148,19 +1055,11 @@ class StubgencSuite(unittest.TestCase):
             """
 
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType("argparse", "")
-        generate_c_function_stub(
-            mod,
-            "test",
-            test,
-            output=output,
-            imports=imports,
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
-        )
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_function_stub("test", test, output=output)
         assert_equal(output, ["def test(arg0: list[Action]) -> list[Action]: ..."])
-        assert_equal(imports, [])
+        assert_equal(gen.get_imports().splitlines(), [])
 
     def test_generate_c_function_same_module_compound(self) -> None:
         """Test that if annotation references type from same module but using full path, no module
@@ -1175,19 +1074,11 @@ class StubgencSuite(unittest.TestCase):
             """
 
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType("argparse", "")
-        generate_c_function_stub(
-            mod,
-            "test",
-            test,
-            output=output,
-            imports=imports,
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
-        )
-        assert_equal(output, ["def test(arg0: Union[Action,None]) -> Tuple[Action,None]: ..."])
-        assert_equal(imports, [])
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_function_stub("test", test, output=output)
+        assert_equal(output, ["def test(arg0: Union[Action, None]) -> Tuple[Action, None]: ..."])
+        assert_equal(gen.get_imports().splitlines(), [])
 
     def test_generate_c_function_other_module_nested(self) -> None:
         """Test that if annotation references type from other module, module will be imported,
@@ -1199,19 +1090,13 @@ class StubgencSuite(unittest.TestCase):
             """
 
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType(self.__module__, "")
-        generate_c_function_stub(
-            mod,
-            "test",
-            test,
-            output=output,
-            imports=imports,
-            known_modules=["foo", "foo.spangle", "bar"],
-            sig_generators=get_sig_generators(parse_options([])),
+        gen = InspectionStubGenerator(
+            mod.__name__, known_modules=["foo", "foo.spangle", "bar"], module=mod
         )
+        gen.generate_function_stub("test", test, output=output)
         assert_equal(output, ["def test(arg0: foo.bar.Action) -> other.Thing: ..."])
-        assert_equal(set(imports), {"import foo", "import other"})
+        assert_equal(gen.get_imports().splitlines(), ["import foo", "import other"])
 
     def test_generate_c_function_no_crash_for_non_str_docstring(self) -> None:
         def test(arg0: str) -> None:
@@ -1220,19 +1105,11 @@ class StubgencSuite(unittest.TestCase):
         test.__doc__ = property(lambda self: "test(arg0: str) -> None")  # type: ignore[assignment]
 
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType(self.__module__, "")
-        generate_c_function_stub(
-            mod,
-            "test",
-            test,
-            output=output,
-            imports=imports,
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
-        )
-        assert_equal(output, ["def test(*args, **kwargs) -> Any: ..."])
-        assert_equal(imports, [])
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_function_stub("test", test, output=output)
+        assert_equal(output, ["def test(*args, **kwargs): ..."])
+        assert_equal(gen.get_imports().splitlines(), [])
 
     def test_generate_c_property_with_pybind11(self) -> None:
         """Signatures included by PyBind11 inside property.fget are read."""
@@ -1247,13 +1124,15 @@ class StubgencSuite(unittest.TestCase):
 
         readwrite_properties: list[str] = []
         readonly_properties: list[str] = []
-        generate_c_property_stub(
+        mod = ModuleType("module", "")  # any module is fine
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_property_stub(
             "attribute",
+            TestClass.__dict__["attribute"],
             TestClass.attribute,
             [],
             readwrite_properties,
             readonly_properties,
-            is_c_property_readonly(TestClass.attribute),
         )
         assert_equal(readwrite_properties, [])
         assert_equal(readonly_properties, ["@property", "def attribute(self) -> str: ..."])
@@ -1273,15 +1152,17 @@ class StubgencSuite(unittest.TestCase):
 
         readwrite_properties: list[str] = []
         readonly_properties: list[str] = []
-        generate_c_property_stub(
+        mod = ModuleType("module", "")  # any module is fine
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_property_stub(
             "attribute",
-            type(TestClass.attribute),
+            TestClass.__dict__["attribute"],
+            TestClass.attribute,
             [],
             readwrite_properties,
             readonly_properties,
-            is_c_property_readonly(TestClass.attribute),
         )
-        assert_equal(readwrite_properties, ["attribute: Any"])
+        assert_equal(readwrite_properties, ["attribute: Incomplete"])
         assert_equal(readonly_properties, [])
 
     def test_generate_c_type_with_single_arg_generic(self) -> None:
@@ -1292,22 +1173,16 @@ class StubgencSuite(unittest.TestCase):
                 """
 
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType(TestClass.__module__, "")
-        generate_c_function_stub(
-            mod,
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_function_stub(
             "test",
             TestClass.test,
             output=output,
-            imports=imports,
-            self_var="self",
-            cls=TestClass,
-            class_name="TestClass",
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
+            class_info=ClassInfo(self_var="self", cls=TestClass, name="TestClass"),
         )
         assert_equal(output, ["def test(self, arg0: List[int]) -> Any: ..."])
-        assert_equal(imports, [])
+        assert_equal(gen.get_imports().splitlines(), [])
 
     def test_generate_c_type_with_double_arg_generic(self) -> None:
         class TestClass:
@@ -1317,22 +1192,16 @@ class StubgencSuite(unittest.TestCase):
                 """
 
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType(TestClass.__module__, "")
-        generate_c_function_stub(
-            mod,
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_function_stub(
             "test",
             TestClass.test,
             output=output,
-            imports=imports,
-            self_var="self",
-            cls=TestClass,
-            class_name="TestClass",
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
+            class_info=ClassInfo(self_var="self", cls=TestClass, name="TestClass"),
         )
-        assert_equal(output, ["def test(self, arg0: Dict[str,int]) -> Any: ..."])
-        assert_equal(imports, [])
+        assert_equal(output, ["def test(self, arg0: Dict[str, int]) -> Any: ..."])
+        assert_equal(gen.get_imports().splitlines(), [])
 
     def test_generate_c_type_with_nested_generic(self) -> None:
         class TestClass:
@@ -1342,22 +1211,16 @@ class StubgencSuite(unittest.TestCase):
                 """
 
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType(TestClass.__module__, "")
-        generate_c_function_stub(
-            mod,
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_function_stub(
             "test",
             TestClass.test,
             output=output,
-            imports=imports,
-            self_var="self",
-            cls=TestClass,
-            class_name="TestClass",
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
+            class_info=ClassInfo(self_var="self", cls=TestClass, name="TestClass"),
         )
-        assert_equal(output, ["def test(self, arg0: Dict[str,List[int]]) -> Any: ..."])
-        assert_equal(imports, [])
+        assert_equal(output, ["def test(self, arg0: Dict[str, List[int]]) -> Any: ..."])
+        assert_equal(gen.get_imports().splitlines(), [])
 
     def test_generate_c_type_with_generic_using_other_module_first(self) -> None:
         class TestClass:
@@ -1367,22 +1230,16 @@ class StubgencSuite(unittest.TestCase):
                 """
 
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType(TestClass.__module__, "")
-        generate_c_function_stub(
-            mod,
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_function_stub(
             "test",
             TestClass.test,
             output=output,
-            imports=imports,
-            self_var="self",
-            cls=TestClass,
-            class_name="TestClass",
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
+            class_info=ClassInfo(self_var="self", cls=TestClass, name="TestClass"),
         )
-        assert_equal(output, ["def test(self, arg0: Dict[argparse.Action,int]) -> Any: ..."])
-        assert_equal(imports, ["import argparse"])
+        assert_equal(output, ["def test(self, arg0: Dict[argparse.Action, int]) -> Any: ..."])
+        assert_equal(gen.get_imports().splitlines(), ["import argparse"])
 
     def test_generate_c_type_with_generic_using_other_module_last(self) -> None:
         class TestClass:
@@ -1392,22 +1249,16 @@ class StubgencSuite(unittest.TestCase):
                 """
 
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType(TestClass.__module__, "")
-        generate_c_function_stub(
-            mod,
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_function_stub(
             "test",
             TestClass.test,
             output=output,
-            imports=imports,
-            self_var="self",
-            cls=TestClass,
-            class_name="TestClass",
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
+            class_info=ClassInfo(self_var="self", cls=TestClass, name="TestClass"),
         )
-        assert_equal(output, ["def test(self, arg0: Dict[str,argparse.Action]) -> Any: ..."])
-        assert_equal(imports, ["import argparse"])
+        assert_equal(output, ["def test(self, arg0: Dict[str, argparse.Action]) -> Any: ..."])
+        assert_equal(gen.get_imports().splitlines(), ["import argparse"])
 
     def test_generate_c_type_with_overload_pybind11(self) -> None:
         class TestClass:
@@ -1422,19 +1273,13 @@ class StubgencSuite(unittest.TestCase):
                 """
 
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType(TestClass.__module__, "")
-        generate_c_function_stub(
-            mod,
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_function_stub(
             "__init__",
             TestClass.__init__,
             output=output,
-            imports=imports,
-            self_var="self",
-            cls=TestClass,
-            class_name="TestClass",
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
+            class_info=ClassInfo(self_var="self", cls=TestClass, name="TestClass"),
         )
         assert_equal(
             output,
@@ -1447,7 +1292,7 @@ class StubgencSuite(unittest.TestCase):
                 "def __init__(self, *args, **kwargs) -> Any: ...",
             ],
         )
-        assert_equal(set(imports), {"from typing import overload"})
+        assert_equal(gen.get_imports().splitlines(), ["from typing import overload"])
 
     def test_generate_c_type_with_overload_shiboken(self) -> None:
         class TestClass:
@@ -1460,19 +1305,18 @@ class StubgencSuite(unittest.TestCase):
                 pass
 
         output: list[str] = []
-        imports: list[str] = []
         mod = ModuleType(TestClass.__module__, "")
-        generate_c_function_stub(
-            mod,
+        gen = InspectionStubGenerator(mod.__name__, known_modules=[mod.__name__], module=mod)
+        gen.generate_function_stub(
             "__init__",
             TestClass.__init__,
             output=output,
-            imports=imports,
-            self_var="self",
-            cls=TestClass,
-            class_name="TestClass",
-            known_modules=[mod.__name__],
-            sig_generators=get_sig_generators(parse_options([])),
+            class_info=ClassInfo(
+                self_var="self",
+                cls=TestClass,
+                name="TestClass",
+                docstring=getattr(TestClass, "__doc__", None),
+            ),
         )
         assert_equal(
             output,
@@ -1483,7 +1327,7 @@ class StubgencSuite(unittest.TestCase):
                 "def __init__(self, arg0: str, arg1: str) -> None: ...",
             ],
         )
-        assert_equal(set(imports), {"from typing import overload"})
+        assert_equal(gen.get_imports().splitlines(), ["from typing import overload"])
 
 
 class ArgSigSuite(unittest.TestCase):
